@@ -1,8 +1,10 @@
 package Spreadsheet::WriteExcel::Big;
 
-
 ###############################################################################
 #
+# Used in conjunction with the big.pl program.
+#
+#
 # Example of how to use extend the  Spreadsheet::WriteExcel 7MB limit with
 # OLE::Storage_Lite http://search.cpan.org/search?dist=OLE-Storage_Lite
 #
@@ -20,7 +22,6 @@ package Spreadsheet::WriteExcel::Big;
 #
 # Copyright 2000, John McNamara, jmcnamara@cpan.org
 #
-# Documentation after __END__
 #
 
 require Exporter;
@@ -61,3 +62,37 @@ sub new {
 
 
 __END__
+
+
+
+=head1 NAME
+
+
+Big - A class for creating Excel files > 7MB.
+
+
+=head1 SYNOPSIS
+
+
+See the documentation for Spreadsheet::WriteExcel.
+
+
+=head1 DESCRIPTION
+
+
+This module is used in conjunction with Spreadsheet::WriteExcel.
+
+
+=head1 AUTHOR
+
+
+John McNamara jmcnamara@cpan.org
+
+
+=head1 COPYRIGHT
+
+
+ MM-MMI, John McNamara.
+
+
+All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
\ No newline at end of file
